# (C) 2025 GoodData Corporation
from pathlib import Path

import pytest
from gooddata_pandas import DataFrameFactory
from gooddata_sdk import CatalogWorkspace
from gooddata_sdk.sdk import GoodDataSdk
from tests_support.vcrpy_utils import get_vcr

gd_vcr = get_vcr()

_current_dir = Path(__file__).parent.absolute()
_fixtures_dir = _current_dir / "fixtures"


def _setup_test_workspace(sdk: GoodDataSdk, test_workspace_id: str, path: Path) -> None:
    """Helper function to set up test workspace with required models.

    Args:
        sdk: GoodData SDK instance
        test_workspace_id: ID of the test workspace
        path: Path to the directory containing model files
    """
    # Check if workspace already exists
    try:
        sdk.catalog_workspace.get_workspace(test_workspace_id)
    except Exception:
        workspace = CatalogWorkspace(workspace_id=test_workspace_id, name=test_workspace_id)
        sdk.catalog_workspace.create_or_update(workspace)

    # Load LDM from disk and put it to the workspace
    ldm_from_disk = sdk.catalog_workspace_content.load_ldm_from_disk(path)
    sdk.catalog_workspace_content.put_declarative_ldm(test_workspace_id, ldm_from_disk, standalone_copy=True)

    # Load Analytics Model from disk and put it to the workspace
    am_from_disk = sdk.catalog_workspace_content.load_analytics_model_from_disk(path)
    sdk.catalog_workspace_content.put_declarative_analytics_model(test_workspace_id, am_from_disk)


@pytest.skip("Requires a complete local GoodData environment with gen_ai service", allow_module_level=True)
@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_created_visualization.yaml"))
def test_dataframe_for_created_visualization(test_config):
    # To recreate the cassette, a complete local GoodData environment with gen_ai service is required.
    # The gen_ai service cannot be included in the already available single docker image due to size constraints.
    # Since in such cases we are interacting with an LLM, the results are not 100% deterministic.

    # Get SDK instance and create test workspace
    sdk = GoodDataSdk.create(host_=test_config["host"], token_=test_config["token"])
    gdf = DataFrameFactory(sdk, test_config["workspace_test"])
    path = _current_dir / "load" / "ai"
    test_workspace_id = test_config["workspace_test"]

    try:
        _setup_test_workspace(sdk, test_workspace_id, path)
        response = sdk.compute.ai_chat(test_workspace_id, "Display the revenue by product")

        df, df_metadata = gdf.for_created_visualization(response)

        assert len(df.columns) == 1
        assert len(df) == 18
        assert df.columns[0] == ("Revenue",)
        assert df.index.names[0] == "Product name"

    finally:
        sdk.catalog_workspace.delete_workspace(test_workspace_id)
        sdk.compute.reset_ai_chat_history(test_workspace_id)



# (C) 2021 GoodData Corporation
from pathlib import Path

import pytest
import yaml


def pytest_addoption(parser):
    default_config_path = Path(__file__).parent / "gd_test_config.yaml"
    parser.addoption(
        "--gd-test-config",
        action="store",
        default=str(default_config_path),
        help="Absolut path to test configuration",
    )


@pytest.fixture(scope="session")
def test_config(request):
    config_path = Path(request.config.getoption("--gd-test-config"))
    with open(config_path) as f:
        config = yaml.safe_load(f)

    return config



# (C) 2021 GoodData Corporation
from pathlib import Path

import pytest
from gooddata_pandas import DataFrameFactory
from gooddata_sdk import Attribute, MetricValueFilter, ObjId, PositiveAttributeFilter
from tests_support.vcrpy_utils import get_vcr

gd_vcr = get_vcr()

_current_dir = Path(__file__).parent.absolute()
_fixtures_dir = _current_dir / "fixtures"


index_types = [
    # reference to the columns key
    "region",
    dict(reg="region"),
    # label_id, i.e. obj id without "label/" prefix - index can reference only attributes
    "region",
    dict(region="region"),
    # object identifier in string form
    "label/region",
    dict(reg="label/region"),
    # Attribute instance
    Attribute(local_id="abcd", label=ObjId(id="region", type="label")),
    dict(region=Attribute(local_id="abcd", label=ObjId(id="region", type="label"))),
    # ObjId instance
    ObjId(id="region", type="label"),
    dict(region=ObjId(id="region", type="label")),
]


@gd_vcr.use_cassette(str(_fixtures_dir / "simple_index_metrics.yaml"))
@pytest.mark.parametrize("index", index_types)
def test_simple_index_metrics(gdf: DataFrameFactory, index):
    df = gdf.indexed(
        index_by=index,
        columns=dict(
            region="label/region",
            category="label/products.category",
            price="fact/price",
        ),
    )

    assert len(df) == 17
    assert len(df.columns) == 3
    assert df.columns[0] == "region"
    assert df.columns[1] == "category"
    assert df.columns[2] == "price"


@gd_vcr.use_cassette(str(_fixtures_dir / "simple_index_metrics_no_duplicate.yaml"))
def test_simple_index_metrics_no_duplicate_index_col(gdf: DataFrameFactory):
    df = gdf.indexed(
        index_by="label/region",
        columns=dict(
            price="fact/price",
            quantity="fact/quantity",
        ),
    )

    assert len(df) == 5
    assert len(df.columns) == 2
    assert df.columns[0] == "price"
    assert df.columns[1] == "quantity"


@gd_vcr.use_cassette(str(_fixtures_dir / "simple_index_metrics_and_label.yaml"))
def test_simple_index_metrics_and_label(gdf: DataFrameFactory):
    columns = {
        "Price": "fact/price",
        "Quantity ($special$%^&)": "fact/quantity",
        "Region code ($special$%^&)": "label/region",
    }
    df = gdf.indexed(
        index_by=dict(reg="label/region"),
        columns=columns,
    )

    assert len(df) == 5
    assert len(df.columns) == 3
    for df_col_name, col_name in zip(df.columns, columns.keys()):
        assert df_col_name == col_name


@gd_vcr.use_cassette(str(_fixtures_dir / "simple_index_filtered_metrics_and_label.yaml"))
def test_simple_index_filtered_metrics_and_label(gdf: DataFrameFactory):
    df = gdf.indexed(
        index_by=dict(reg="label/region"),
        columns=dict(
            price="fact/price",
            quantity="fact/quantity",
            category="label/products.category",
        ),
        filter_by=[
            # Label referenced by localIdentifier
            PositiveAttributeFilter(label="reg", values=["Midwest"]),
            # Label referenced by full ID
            PositiveAttributeFilter(label="label/region", values=["Midwest"]),
            # Label referenced by ObjId
            PositiveAttributeFilter(label=ObjId(id="region", type="label"), values=["Midwest"]),
            # label referenced by index in columns
            PositiveAttributeFilter(label="category", values=["Clothing"]),
            MetricValueFilter(metric="price", operator="GREATER_THAN", values=100),
        ],
    )

    assert len(df) == 1
    assert len(df.columns) == 3
    assert df.columns[0] == "price"
    assert df.columns[1] == "quantity"
    assert df.columns[2] == "category"


@gd_vcr.use_cassette(str(_fixtures_dir / "multi_index_metrics.yaml"))
def test_multi_index_metrics(gdf: DataFrameFactory):
    df = gdf.indexed(
        index_by=dict(reg="label/region", category="label/products.category"),
        columns=dict(order_amount="metric/order_amount", order_count="metric/amount_of_orders"),
    )

    assert len(df) == 17
    assert len(df.columns) == 2
    assert df.index.names[0] == "reg"
    assert df.index.names[1] == "category"
    assert df.columns[0] == "order_amount"
    assert df.columns[1] == "order_count"


@gd_vcr.use_cassette(str(_fixtures_dir / "multi_index_metrics_and_label.yaml"))
def test_multi_index_metrics_and_label(gdf: DataFrameFactory):
    df = gdf.indexed(
        index_by=dict(reg="label/region", category="label/products.category"),
        columns=dict(
            order_amount="metric/order_amount",
            order_count="metric/amount_of_orders",
            state="label/state",
        ),
    )

    assert len(df) > 17
    assert len(df.columns) == 3
    assert df.index.names[0] == "reg"
    assert df.index.names[1] == "category"
    assert df.columns[0] == "order_amount"
    assert df.columns[1] == "order_count"
    assert df.columns[2] == "state"


@gd_vcr.use_cassette(str(_fixtures_dir / "multi_index_filtered_metrics_and_label.yaml"))
def test_multi_index_filtered_metrics_and_label(gdf: DataFrameFactory):
    df = gdf.indexed(
        index_by=dict(reg="label/region", category="label/products.category"),
        columns=dict(
            order_amount="metric/order_amount",
            order_count="metric/amount_of_orders",
            state="label/state",
        ),
        filter_by=[
            PositiveAttributeFilter(label="reg", values=["Northeast"]),
            MetricValueFilter(metric="order_count", operator="GREATER_THAN", values=50),
        ],
    )

    assert len(df) == 3
    assert len(df.columns) == 3
    assert df.index.names[0] == "reg"
    assert df.index.names[1] == "category"
    assert df.columns[0] == "order_amount"
    assert df.columns[1] == "order_count"
    assert df.columns[2] == "state"


@gd_vcr.use_cassette(str(_fixtures_dir / "multi_index_filtered_metrics_and_label_reuse.yaml"))
def test_multi_index_filtered_metrics_and_label_reuse(gdf: DataFrameFactory):
    # note here: if a single label is reused in both index and columns, that label will be used in computation
    # just once. the first-found occurrence will be used in computation. local id of the attribute will be
    # the key in the dict where the label was first found
    #
    # this has implications when referencing label by local id
    df = gdf.indexed(
        index_by=dict(reg="label/region", category="label/products.category"),
        columns=dict(
            order_amount="metric/order_amount",
            order_count="metric/amount_of_orders",
            reg="label/region",
        ),
        filter_by=[PositiveAttributeFilter(label="reg", values=["Midwest"])],
    )

    assert len(df) == 4
    assert len(df.columns) == 3
    assert df.index.names[0] == "reg"
    assert df.index.names[1] == "category"
    assert df.columns[0] == "order_amount"
    assert df.columns[1] == "order_count"
    assert df.columns[2] == "reg"


@gd_vcr.use_cassette(str(_fixtures_dir / "empty_indexed_dataframe.yaml"))
def test_empty_indexed_dataframe(gdf: DataFrameFactory):
    df = gdf.indexed(
        index_by="label/product_name",
        columns=dict(
            amount_of_top_customers="metric/amount_of_top_customers",
            total_revenue="metric/total_revenue-no_filters",
        ),
    )

    assert df.empty
    assert df.columns[0] == "amount_of_top_customers"
    assert df.columns[1] == "total_revenue"



# (C) 2022 GoodData Corporation
from pathlib import Path
from typing import Optional

import pytest
from gooddata_pandas import DataFrameFactory
from gooddata_sdk import (
    Attribute,
    ExecutionDefinition,
    ObjId,
    ResultSizeBytesLimitExceeded,
    ResultSizeDimensionsLimitsExceeded,
    SimpleMetric,
    TableDimension,
    TotalDefinition,
    TotalDimension,
)
from tests_support.vcrpy_utils import get_vcr

gd_vcr = get_vcr()

_current_dir = Path(__file__).parent.absolute()
_fixtures_dir = _current_dir / "fixtures"


def _run_and_validate_results(
    gdf: DataFrameFactory,
    exec_def: ExecutionDefinition,
    expected: tuple[int, int],
    expected_row_totals: Optional[list[list[int]]] = None,
    expected_column_totals: Optional[list[list[int]]] = None,
    page_size: int = 100,
    optimized: bool = False,
) -> str:
    # generate dataframe from exec_def
    result, result_metadata = gdf.for_exec_def(exec_def=exec_def, page_size=page_size)
    assert result.values.shape == expected

    # use result ID from computation above and generate dataframe just from it
    result_from_result_id, result_metadata_from_result_id = gdf.for_exec_result_id(
        result_id=result_metadata.execution_response.result_id, page_size=page_size, optimized=optimized
    )

    if expected_row_totals is not None:
        assert result_metadata_from_result_id.row_totals_indexes == expected_row_totals
    if expected_column_totals is not None:
        assert result_metadata_from_result_id.column_totals_indexes == expected_column_totals

    assert result_from_result_id.values.shape == expected

    # compare dataframes generated using both methods above
    assert result.to_string() == result_from_result_id.to_string()

    return result_metadata.execution_response.result_id


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_exec_def_two_dim1.yaml"))
@pytest.mark.parametrize("optimized", [True, False])
def test_dataframe_for_exec_def_two_dim1(test_config, gdf: DataFrameFactory, optimized: bool):
    exec_def = ExecutionDefinition(
        attributes=[
            Attribute(local_id="region", label="region"),
            Attribute(local_id="state", label="state"),
            Attribute(local_id="product_category", label="products.category"),
        ],
        metrics=[
            SimpleMetric(local_id="price", item=ObjId(id="price", type="fact")),
            SimpleMetric(local_id="order_amount", item=ObjId(id="order_amount", type="metric")),
        ],
        filters=[],
        dimensions=[
            TableDimension(item_ids=["state", "region"]),
            TableDimension(item_ids=["product_category", "measureGroup"]),
        ],
    )
    exec_result_id = _run_and_validate_results(gdf=gdf, exec_def=exec_def, expected=(48, 8), optimized=optimized)

    # check also label overrides
    overrides = {
        "labels": {
            "state": {"title": "STATE LABEL"},
        },
        "metrics": {
            "price": {"title": "PRICE LABEL"},
        },
    }
    result, _ = gdf.for_exec_result_id(exec_result_id, label_overrides=overrides, optimized=optimized)
    assert result.to_string().find(overrides["labels"]["state"]["title"]) == 262
    assert result.to_string().find(overrides["metrics"]["price"]["title"]) == 162


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_exec_def_dimensions_limits_failure.yaml"))
def test_dataframe_for_exec_def_dimensions_limits_failure(test_config, gdf: DataFrameFactory):
    exec_def = ExecutionDefinition(
        attributes=[
            Attribute(local_id="region", label="region"),
            Attribute(local_id="state", label="state"),
            Attribute(local_id="product_category", label="products.category"),
        ],
        metrics=[
            SimpleMetric(local_id="price", item=ObjId(id="price", type="fact")),
            SimpleMetric(local_id="order_amount", item=ObjId(id="order_amount", type="metric")),
        ],
        filters=[],
        dimensions=[
            TableDimension(item_ids=["state", "region"]),
            TableDimension(item_ids=["product_category", "measureGroup"]),
        ],
    )

    result_size_dimensions_limits = (1, 1)
    exception = None
    try:
        gdf.for_exec_def(exec_def=exec_def, result_size_dimensions_limits=result_size_dimensions_limits)
    except ResultSizeDimensionsLimitsExceeded as e:
        exception = e

    assert type(exception) is ResultSizeDimensionsLimitsExceeded
    assert exception.result_size_dimensions_limits == result_size_dimensions_limits
    assert exception.first_violating_index == 0


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_exec_def_bytes_limits_failure.yaml"))
def test_dataframe_for_exec_def_bytes_limits_failure(test_config, gdf: DataFrameFactory):
    exec_def = ExecutionDefinition(
        attributes=[
            Attribute(local_id="region", label="region"),
            Attribute(local_id="state", label="state"),
            Attribute(local_id="product_category", label="products.category"),
        ],
        metrics=[
            SimpleMetric(local_id="price", item=ObjId(id="price", type="fact")),
            SimpleMetric(local_id="order_amount", item=ObjId(id="order_amount", type="metric")),
        ],
        filters=[],
        dimensions=[
            TableDimension(item_ids=["state", "region"]),
            TableDimension(item_ids=["product_category", "measureGroup"]),
        ],
    )

    result_size_bytes_limit = 2047
    exception = None
    try:
        gdf.for_exec_def(exec_def=exec_def, result_size_bytes_limit=result_size_bytes_limit)
    except ResultSizeBytesLimitExceeded as e:
        exception = e

    assert type(exception) is ResultSizeBytesLimitExceeded
    assert exception.result_size_bytes_limit == result_size_bytes_limit


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_exec_def_two_dim2.yaml"))
@pytest.mark.parametrize("optimized", [True, False])
def test_dataframe_for_exec_def_two_dim2(gdf: DataFrameFactory, optimized: bool):
    exec_def = ExecutionDefinition(
        attributes=[
            Attribute(local_id="region", label="region"),
            Attribute(local_id="state", label="state"),
            Attribute(local_id="product_category", label="products.category"),
        ],
        metrics=[
            SimpleMetric(local_id="price", item=ObjId(id="price", type="fact")),
            SimpleMetric(local_id="order_amount", item=ObjId(id="order_amount", type="metric")),
        ],
        filters=[],
        dimensions=[
            TableDimension(item_ids=["region", "state", "product_category"]),
            TableDimension(item_ids=["measureGroup"]),
        ],
    )
    _run_and_validate_results(gdf=gdf, exec_def=exec_def, expected=(182, 2), optimized=optimized)


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_exec_def_two_dim3.yaml"))
@pytest.mark.parametrize("optimized", [True, False])
def test_dataframe_for_exec_def_two_dim3(gdf: DataFrameFactory, optimized: bool):
    exec_def = ExecutionDefinition(
        attributes=[
            Attribute(local_id="region", label="region"),
            Attribute(local_id="state", label="state"),
            Attribute(local_id="product_category", label="products.category"),
        ],
        metrics=[
            SimpleMetric(local_id="price", item=ObjId(id="price", type="fact")),
            SimpleMetric(local_id="order_amount", item=ObjId(id="order_amount", type="metric")),
        ],
        filters=[],
        dimensions=[
            TableDimension(item_ids=["product_category"]),
            TableDimension(item_ids=["region", "state", "measureGroup"]),
        ],
    )
    _run_and_validate_results(gdf=gdf, exec_def=exec_def, expected=(4, 96), optimized=optimized)


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_exec_def_totals1.yaml"))
@pytest.mark.parametrize("optimized", [True, False])
def test_dataframe_for_exec_def_totals1(gdf: DataFrameFactory, optimized: bool):
    """
    Execution with column totals; the row dimension has single label
    """
    exec_def = ExecutionDefinition(
        attributes=[
            Attribute(local_id="region", label="region"),
            Attribute(local_id="state", label="state"),
            Attribute(local_id="product_category", label="products.category"),
        ],
        metrics=[
            SimpleMetric(local_id="price", item=ObjId(id="price", type="fact")),
            SimpleMetric(local_id="order_amount", item=ObjId(id="order_amount", type="metric")),
        ],
        filters=[],
        dimensions=[
            TableDimension(item_ids=["product_category"]),
            TableDimension(item_ids=["region", "state", "measureGroup"]),
        ],
        totals=[
            TotalDefinition(
                local_id="grand_total1",
                aggregation="sum",
                total_dims=[TotalDimension(idx=1, items=["region", "state", "measureGroup"])],
                metric_local_id="price",
            ),
            TotalDefinition(
                local_id="grand_total2",
                aggregation="max",
                total_dims=[TotalDimension(idx=1, items=["region", "state", "measureGroup"])],
                metric_local_id="order_amount",
            ),
        ],
    )
    _run_and_validate_results(
        gdf=gdf, exec_def=exec_def, expected=(6, 96), expected_row_totals=[[4, 5]], optimized=optimized
    )


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_exec_def_totals2.yaml"))
@pytest.mark.parametrize("optimized", [True, False])
def test_dataframe_for_exec_def_totals2(gdf: DataFrameFactory, optimized: bool):
    """
    Execution with column totals; the row dimension have two labels; this exercises that the index is
    padded appropriately
    """
    exec_def = ExecutionDefinition(
        attributes=[
            Attribute(local_id="region", label="region"),
            Attribute(local_id="state", label="state"),
            Attribute(local_id="product_category", label="products.category"),
        ],
        metrics=[
            SimpleMetric(local_id="price", item=ObjId(id="price", type="fact")),
            SimpleMetric(local_id="order_amount", item=ObjId(id="order_amount", type="metric")),
        ],
        filters=[],
        dimensions=[
            TableDimension(item_ids=["region", "product_category"]),
            TableDimension(item_ids=["state", "measureGroup"]),
        ],
        totals=[
            TotalDefinition(
                local_id="grand_total1",
                aggregation="sum",
                total_dims=[TotalDimension(idx=1, items=["state", "measureGroup"])],
                metric_local_id="price",
            ),
            TotalDefinition(
                local_id="grand_total2",
                aggregation="max",
                total_dims=[TotalDimension(idx=1, items=["state", "measureGroup"])],
                metric_local_id="order_amount",
            ),
        ],
    )
    _run_and_validate_results(
        gdf=gdf, exec_def=exec_def, expected=(19, 96), expected_row_totals=[[17, 18], [17, 18]], optimized=optimized
    )


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_exec_def_totals3.yaml"))
@pytest.mark.parametrize("optimized", [True, False])
def test_dataframe_for_exec_def_totals3(gdf: DataFrameFactory, optimized: bool):
    """
    Execution with row totals; the column dimension has single label.
    """
    exec_def = ExecutionDefinition(
        attributes=[
            Attribute(local_id="region", label="region"),
            Attribute(local_id="state", label="state"),
            Attribute(local_id="product_category", label="products.category"),
        ],
        metrics=[
            SimpleMetric(local_id="price", item=ObjId(id="price", type="fact")),
            SimpleMetric(local_id="order_amount", item=ObjId(id="order_amount", type="metric")),
        ],
        filters=[],
        dimensions=[
            TableDimension(item_ids=["region", "state", "measureGroup"]),
            TableDimension(item_ids=["product_category"]),
        ],
        totals=[
            TotalDefinition(
                local_id="grand_total1",
                aggregation="sum",
                total_dims=[TotalDimension(idx=0, items=["region", "state", "measureGroup"])],
                metric_local_id="price",
            ),
            TotalDefinition(
                local_id="grand_total2",
                aggregation="max",
                total_dims=[TotalDimension(idx=0, items=["region", "state", "measureGroup"])],
                metric_local_id="order_amount",
            ),
        ],
    )
    _run_and_validate_results(gdf=gdf, exec_def=exec_def, expected=(96, 6), optimized=optimized)


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_exec_def_totals4.yaml"))
@pytest.mark.parametrize("optimized", [True, False])
def test_dataframe_for_exec_def_totals4(gdf: DataFrameFactory, optimized: bool):
    """
    Execution with row totals; the column dimension have two label.
    """
    exec_def = ExecutionDefinition(
        attributes=[
            Attribute(local_id="region", label="region"),
            Attribute(local_id="state", label="state"),
            Attribute(local_id="product_category", label="products.category"),
        ],
        metrics=[
            SimpleMetric(local_id="price", item=ObjId(id="price", type="fact")),
            SimpleMetric(local_id="order_amount", item=ObjId(id="order_amount", type="metric")),
        ],
        filters=[],
        dimensions=[
            TableDimension(item_ids=["state", "measureGroup"]),
            TableDimension(item_ids=["region", "product_category"]),
        ],
        totals=[
            TotalDefinition(
                local_id="grand_total1",
                aggregation="sum",
                total_dims=[TotalDimension(idx=0, items=["state", "measureGroup"])],
                metric_local_id="price",
            ),
            TotalDefinition(
                local_id="grand_total2",
                aggregation="max",
                total_dims=[TotalDimension(idx=0, items=["state", "measureGroup"])],
                metric_local_id="order_amount",
            ),
        ],
    )
    _run_and_validate_results(
        gdf=gdf, exec_def=exec_def, expected=(96, 19), optimized=optimized, expected_column_totals=[[17, 18], [17, 18]]
    )


# TODO - not implemented yet
# def test_dataframe_for_exec_def_totals5(gdf: DataFrameFactory):
#     """
#     Execution with multiple measures row totals; the columns have labels of each measure.
#     """
#     exec_def = ExecutionDefinition(
#         attributes=[
#             Attribute(local_id="a_region", label="region"),
#             Attribute(local_id="a_cat", label="products.category"),
#         ],
#         metrics=[
#             SimpleMetric(local_id="m_price", item=ObjId(id="price", type="fact")),
#             SimpleMetric(local_id="m_quantity", item=ObjId(id="quantity", type="fact")),
#         ],
#         filters=[],
#         dimensions=[
#             TableDimension(item_ids=["a_region"]),
#             TableDimension(item_ids=["a_cat", "measureGroup"]),
#         ],
#         totals=[
#             TotalDefinition(
#                 local_id="grand_total1",
#                 aggregation="sum",
#                 total_dims=[TotalDimension(idx=1, items=["measureGroup"]), TotalDimension(idx=0, items=["a_region"])],
#                 metric_local_id="m_price",
#             ),
#             TotalDefinition(
#                 local_id="grand_total2",
#                 aggregation="sum",
#                 total_dims=[TotalDimension(idx=1, items=["measureGroup"]), TotalDimension(idx=0, items=["a_region"])],
#                 metric_local_id="m_quantity",
#             ),
#         ],
#     )
#     _run_and_validate_results(gdf=gdf, exec_def=exec_def, expected=(?, ?))


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_exec_def_one_dim1.yaml"))
@pytest.mark.parametrize("optimized", [True, False])
def test_dataframe_for_exec_def_one_dim1(gdf: DataFrameFactory, optimized: bool):
    exec_def = ExecutionDefinition(
        attributes=[
            Attribute(local_id="region", label="region"),
            Attribute(local_id="state", label="state"),
            Attribute(local_id="product_category", label="products.category"),
        ],
        metrics=[
            SimpleMetric(local_id="price", item=ObjId(id="price", type="fact")),
            SimpleMetric(local_id="order_amount", item=ObjId(id="order_amount", type="metric")),
        ],
        filters=[],
        dimensions=[TableDimension(item_ids=["region", "state", "product_category", "measureGroup"])],
    )
    # TODO: remove page_size=500 once UNI-591 is resolved
    _run_and_validate_results(gdf=gdf, exec_def=exec_def, expected=(364, 1), page_size=500, optimized=optimized)


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_exec_def_one_dim2.yaml"))
@pytest.mark.parametrize("optimized", [True, False])
def test_dataframe_for_exec_def_one_dim2(gdf: DataFrameFactory, optimized: bool):
    exec_def = ExecutionDefinition(
        attributes=[
            Attribute(local_id="region", label="region"),
            Attribute(local_id="state", label="state"),
            Attribute(local_id="product_category", label="products.category"),
        ],
        metrics=[
            SimpleMetric(local_id="price", item=ObjId(id="price", type="fact")),
            SimpleMetric(local_id="order_amount", item=ObjId(id="order_amount", type="metric")),
        ],
        filters=[],
        dimensions=[
            TableDimension(item_ids=[]),
            TableDimension(item_ids=["region", "state", "product_category", "measureGroup"]),
        ],
    )
    _run_and_validate_results(gdf=gdf, exec_def=exec_def, expected=(1, 364), optimized=optimized)



# (C) 2021 GoodData Corporation
from pathlib import Path

from gooddata_pandas import SeriesFactory
from gooddata_sdk import ObjId, PositiveAttributeFilter
from numpy import float64
from tests_support.vcrpy_utils import get_vcr

gd_vcr = get_vcr()

_current_dir = Path(__file__).parent.absolute()
_fixtures_dir = _current_dir / "fixtures"


@gd_vcr.use_cassette(str(_fixtures_dir / "not_indexed_metric_series.yaml"))
def test_not_indexed_metric_series(gds: SeriesFactory):
    series = gds.not_indexed(data_by="fact/price")

    # having metric with no granularity will return series with single item
    assert len(series) == 1
    assert series.values.dtype == float64


@gd_vcr.use_cassette(str(_fixtures_dir / "not_indexed_label_series.yaml"))
def test_not_index_label_series(gds: SeriesFactory):
    series = gds.not_indexed(data_by="label/region")

    assert len(series) == 5
    assert series[0] == "Midwest"
    assert series[4] == "West"


@gd_vcr.use_cassette(str(_fixtures_dir / "not_indexed_metric_series_with_granularity.yaml"))
def test_not_indexed_metric_series_with_granularity(gds: SeriesFactory):
    series = gds.not_indexed(
        granularity=dict(reg="label/region"),
        data_by="fact/price",
    )

    assert len(series) == 5
    assert series.values.dtype == float64


@gd_vcr.use_cassette(str(_fixtures_dir / "not_indexed_label_series_with_granularity.yaml"))
def test_not_index_label_series_with_granularity(gds: SeriesFactory):
    series = gds.not_indexed(
        granularity=dict(reg="label/region"),
        data_by="label/products.category",
    )

    # just a rough rub; getting series of products on the region granularity leads to series containing
    # an assorted listing of products used in the different regions. there is 1:N relationship there.
    assert len(series) == 17


@gd_vcr.use_cassette(str(_fixtures_dir / "not_indexed_filtered_metric_series.yaml"))
def test_not_indexed_filtered_metric_series(gds: SeriesFactory):
    # price across all regions
    not_filtered_series = gds.not_indexed(data_by="fact/price")
    # price just in Unknown
    filtered_series = gds.not_indexed(
        data_by="fact/price",
        filter_by=PositiveAttributeFilter(label=ObjId(type="label", id="region"), values=["Unknown"]),
    )

    # having metric with no granularity will return series with single item
    assert len(filtered_series) == 1
    # the filtered crime rate should be lower than rate in all regions
    assert filtered_series[0] < not_filtered_series[0]



# (C) 2021 GoodData Corporation
import pytest
from gooddata_pandas import DataFrameFactory, GoodPandas


@pytest.fixture
def gdf(test_config) -> DataFrameFactory:
    gdpd = GoodPandas(host=test_config["host"], token=test_config["token"])
    return gdpd.data_frames(test_config["workspace"])



# (C) 2021 GoodData Corporation
from pathlib import Path

from gooddata_pandas import DataFrameFactory
from tests_support.vcrpy_utils import get_vcr

gd_vcr = get_vcr()

_current_dir = Path(__file__).parent.absolute()
_fixtures_dir = _current_dir / "fixtures"


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_visualization_date.yaml"))
def test_dataframe_for_visualization_date(gdf: DataFrameFactory):
    # 2 metrics grouped by date dimension with data for last 12 months
    # exact numbers cannot be checked as date data are changed each AIO build
    df = gdf.for_visualization(visualization_id="customers_trend")

    assert len(df) == 12
    assert len(df.index.names) == 1
    assert df.index.name == "date.month"
    assert df.index.dtype.name == "datetime64[ns]"
    assert len(df.columns) == 2
    assert df.columns[0] == "amount_of_active_customers"
    assert df.columns[1] == "revenue_per_customer"


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_visualization.yaml"))
def test_dataframe_for_visualization(gdf: DataFrameFactory):
    # 4 metrics grouped by 2 attributes, filters are set to all
    df = gdf.for_visualization(visualization_id="revenue_and_quantity_by_product_and_category")

    assert df.index.names[0] == "products.category"
    assert df.index.names[1] == "product_name"
    assert df.columns[0] == "quantity"
    assert df.columns[1] == "price"
    assert df.columns[2] == "percent_revenue_in_category"
    assert df.columns[3] == "revenue"


@gd_vcr.use_cassette(str(_fixtures_dir / "dataframe_for_visualization_no_index.yaml"))
def test_dataframe_for_visualization_no_index(gdf: DataFrameFactory):
    # 4 metrics grouped by 2 attributes, filters are set to all
    df = gdf.for_visualization(visualization_id="revenue_and_quantity_by_product_and_category", auto_index=False)

    assert df.columns[0] == "products.category"
    assert df.columns[1] == "product_name"
    assert df.columns[2] == "quantity"
    assert df.columns[3] == "price"
    assert df.columns[4] == "percent_revenue_in_category"
    assert df.columns[5] == "revenue"



# (C) 2023 GoodData Corporation
from __future__ import annotations

from pathlib import Path
from typing import Any, Union

import pytest
import yaml
from gooddata_pandas import GoodPandas

_current_dir = Path(__file__).parent.absolute()
PROFILES_PATH = _current_dir / "profiles" / "profiles.yaml"


def load_profiles_content(path: Union[str, Path]) -> dict:
    with open(path, encoding="utf-8") as f:
        return yaml.safe_load(f)


def are_same_check(profile_data: dict[str, Any], good_pandas: GoodPandas):
    assert profile_data["host"] == good_pandas.sdk.client._hostname
    assert profile_data["token"] == good_pandas.sdk.client._token
    if "custom_headers" in profile_data:
        assert profile_data["custom_headers"] == good_pandas.sdk.client._custom_headers


@pytest.mark.parametrize(
    "profile",
    [
        "custom",
        "default",
        "correct_1",
        "correct_2",
        "correct_3",
        "correct_4",
    ],
)
def test_legacy_config(profile):
    good_pandas = GoodPandas.create_from_profile(profile=profile, profiles_path=PROFILES_PATH)
    data = load_profiles_content(PROFILES_PATH)
    are_same_check(data[profile], good_pandas)


def test_wrong_profile():
    profile = "wrong"
    with pytest.raises(ValueError):
        GoodPandas.create_from_profile(profile=profile, profiles_path=PROFILES_PATH)



# (C) 2021 GoodData Corporation
from pathlib import Path

from gooddata_pandas import SeriesFactory
from gooddata_sdk import PositiveAttributeFilter
from numpy import float64
from tests_support.vcrpy_utils import get_vcr

gd_vcr = get_vcr()

_current_dir = Path(__file__).parent.absolute()
_fixtures_dir = _current_dir / "fixtures"


@gd_vcr.use_cassette(str(_fixtures_dir / "simple_index_metric_series.yaml"))
def test_simple_index_metric_series(gds: SeriesFactory):
    series = gds.indexed(
        index_by=dict(reg="label/region"),
        data_by="fact/price",
    )

    assert len(series) == 5
    assert series.values.dtype == float64


@gd_vcr.use_cassette(str(_fixtures_dir / "simple_index_label_series.yaml"))
def test_simple_index_label_series(gds: SeriesFactory):
    series = gds.indexed(
        index_by=dict(reg="label/region"),
        data_by="label/region",
    )

    assert len(series) == 5
    assert series["Midwest"] == "Midwest"


@gd_vcr.use_cassette(str(_fixtures_dir / "simple_index_filtered_series.yaml"))
def test_simple_index_filtered_series(gds: SeriesFactory):
    series = gds.indexed(
        index_by=dict(reg="label/region"),
        data_by="label/products.category",
        filter_by=[
            PositiveAttributeFilter(label="label/region", values=["Midwest"]),
            PositiveAttributeFilter(label="label/products.category", values=["Clothing"]),
        ],
    )

    assert len(series) == 1
    assert series["Midwest"] == "Clothing"


@gd_vcr.use_cassette(str(_fixtures_dir / "multi_index_metric_series.yaml"))
def test_multi_index_metric_series(gds: SeriesFactory):
    series = gds.indexed(
        index_by=dict(reg="label/region", category="label/products.category"),
        data_by="fact/price",
    )

    assert len(series) == 17
    assert len(series["Northeast"]) == 4
    assert len(series["Unknown"]) == 1
    assert series.values.dtype == float64


@gd_vcr.use_cassette(str(_fixtures_dir / "multi_index_filtered_series.yaml"))
def test_multi_index_filtered_series(gds: SeriesFactory):
    series = gds.indexed(
        index_by=dict(reg="label/region", category="label/products.category"),
        data_by="fact/price",
        filter_by=PositiveAttributeFilter(label="category", values=["Clothing"]),
    )

    assert len(series) == 5

    index_list = series.keys().to_list()
    index_list.sort()
    assert index_list == [
        ("Midwest", "Clothing"),
        ("Northeast", "Clothing"),
        ("South", "Clothing"),
        ("Unknown", "Clothing"),
        ("West", "Clothing"),
    ]
    assert len(series["Midwest"]) == 1
    assert series.values.dtype == float64
